#import "OpenCVCalcOpticalFlowPyrLK.h"

@implementation OpenCVCalcOpticalFlowPyrLK : QCPatch

+(BOOL)isSafe
{
	return YES;
}

+ (BOOL)allowsSubpatchesWithIdentifier:(id)fp8
{
	return NO;
}

- (id)initWithIdentifier:(id)fp8
{
	if(self = [super initWithIdentifier:fp8])
	{
		[inputEpsilon setMinDoubleValue: 0.0];
		[inputEpsilon setMaxDoubleValue: 10.0];
		[inputEpsilon setDoubleValue: 0.03];
		[inputMaxIterations setMaxIndexValue: 50];
		[inputMaxIterations setIndexValue: 20];
		[inputWindowSize setIndexValue: 10];
		[inputImage setType: OPENCV_IMAGE];
	}

	return self;
}

- (void)dealloc
{
	if(autoPrevFrame)
	{
		cvReleaseImage(&autoPrevFrame);
		autoPrevFrame = NULL;
	}
	if(pyramid)
	{
		cvReleaseImage(&pyramid);
		pyramid = NULL;
	}
	if(prevPyramid)
	{
		cvReleaseImage(&prevPyramid);
		prevPyramid = NULL;
	}
	[super dealloc];
}

- (BOOL)execute:(QCOpenGLContext *)context time:(double)time arguments:(NSDictionary *)arguments
{
	IplImage *frame = [[inputImage cvObjectValue] value];
	
	if(frame)	// got a frame, do stuff
	{
		if(frame->nChannels > 1)
			return YES;	// this isn't possible for opticalFlow, so we just bail before it gets ugly.
		if([inputUseInputPointsSignal booleanValue] && !oldUseInputPointsSignal)
		{
			unsigned int i;
			QCStructure *inputStruct = [inputPoints structureValue];
			if(pointCount != [inputStruct count])	// this is really expensive.....  infrequent though
			{
				if(points)
					free(points);
				if(prevPoints)
					free(prevPoints);
				if(pointStatus)
					free(pointStatus);
				//[pointStruct release];
				points = NULL;
				prevPoints = NULL;
				pointStatus = NULL;
				pointCount = [inputStruct count];
				if(pointCount)
				{
					points = (CvPoint2D32f*)malloc(sizeof(CvPoint2D32f)*pointCount);
					prevPoints = (CvPoint2D32f*)malloc(sizeof(CvPoint2D32f)*pointCount);
					pointStatus = (char*)malloc(pointCount * sizeof(char));
					//pointStruct = [[QCStructure alloc] init];
				}
				else
				{
					points = NULL;
					prevPoints = NULL;
					pointStatus = NULL;
					//pointStruct = nil;
				}
			}
			for(i=0;i<pointCount;i++)
			{
				QCStructure *inPoint = [inputStruct memberForKey:[NSString stringWithFormat:@"Point%04i",i]];
				prevPoints[i].x = QC2CV([[inPoint memberForKey:@"X"] doubleValue],frame->width);
				prevPoints[i].y = QC2CV([[inPoint memberForKey:@"Y"] doubleValue] * frame->width/frame->height,frame->height);
			}
		}
		// add additional points mid-flight (but not if we've already imported in the import above)
		else if([inputAddInputPointsSignal booleanValue] && !oldAddInputPointsSignal)
		{
			unsigned int i;
			QCStructure *inputStruct = [inputPoints structureValue];
			if(inputStruct)
			{
				unsigned int count, oldPointCount = pointCount;
				count = [inputStruct count];
				pointCount += count;
				points = (CvPoint2D32f*)realloc(points, sizeof(CvPoint2D32f)*pointCount);
				prevPoints = (CvPoint2D32f*)realloc(prevPoints, sizeof(CvPoint2D32f)*pointCount);
				pointStatus = (char*)realloc(pointStatus, sizeof(char)*pointCount);
				for(i=0; i < count; i++)
				{
					QCStructure *inPoint = [inputStruct memberForKey:[NSString stringWithFormat:@"Point%04i",i]];
					prevPoints[oldPointCount + i].x = QC2CV([[inPoint memberForKey:@"X"] doubleValue],frame->width);
					prevPoints[oldPointCount + i].y = QC2CV([[inPoint memberForKey:@"Y"] doubleValue] * frame->width/frame->height,frame->height);
				}
			}
		}
		
		if(pointCount > 0)	// if there aren't any points, it doesn't count as a signal :)
		{
			oldUseInputPointsSignal = [inputUseInputPointsSignal booleanValue];
			oldAddInputPointsSignal = [inputAddInputPointsSignal booleanValue];
		}
		else // no points, nothing to do
		{
			[outputPoints setStructureValue:nil];
			return YES;
		}
		
		if(pyramid == NULL || pyramid->width != frame->width || pyramid->height != frame->height)
		{
			//NSLog(@"Generating Pyramids @ %f",time);
			if(pyramid != NULL)
			{
				cvReleaseImage(&pyramid);
				cvReleaseImage(&prevPyramid);
			}
			pyramid = cvCreateImage(cvGetSize(frame), 8, 1);
			prevPyramid = cvCreateImage(cvGetSize(frame), 8, 1);
		}
		//else
		//	NSLog(@"Not generating pyramids (%x) @ %f",pyramid, time);
		/*if(prevFrame)	// user-supplied prev frame (never happens yet)
		{
			unsigned int i;
			//NSLog(@"Flowing @ %f %x %x!",time, prevPoints, points);
			cvCalcOpticalFlowPyrLK( prevFrame, frame, prevPyramid, pyramid,
                         prevPoints, points,
                         pointCount, cvSize([inputWindowSize indexValue],[inputWindowSize indexValue]), 3, pointStatus,
                         NULL, cvTermCriteria (CV_TERMCRIT_ITER|CV_TERMCRIT_EPS,
                               [inputMaxIterations indexValue], [inputEpsilon doubleValue]), 0);
			double nan = 0.0;
			nan /= nan;
			[pointStruct removeAllMembers];
			for(i=0;i<pointCount;++i)
			{
				if(pointStatus[i])
				{
					[pointStruct addMember:[NSNumber numberWithDouble:points[i].x] forKey:[NSString stringWithFormat:@"X%i",i]];
					[pointStruct addMember:[NSNumber numberWithDouble:points[i].y] forKey:[NSString stringWithFormat:@"Y%i",i]];
				}
				else
				{
					[pointStruct addMember:[NSNumber numberWithDouble:nan] forKey:[NSString stringWithFormat:@"X%i",i]];
					[pointStruct addMember:[NSNumber numberWithDouble:nan] forKey:[NSString stringWithFormat:@"Y%i",i]];
				}
			}
			// swap all the frame data (pyramid, frame, points)
			void *tmp = points;
			points = prevPoints;
			prevPoints = tmp;
			tmp = pyramid;
			pyramid = prevPyramid;
			prevPyramid = tmp;
			[outputPoints setStructureValue: pointStruct];
		}
		else*/	// no user-supplied prev frame, make/use autogenerated one
		{
			if(autoPrevFrame && autoPrevFrame->width == frame->width && autoPrevFrame->height == frame->height)
			{	
				unsigned int i;
				if([inputWindowSize indexValue] < 2)
					[inputWindowSize setIndexValue:2];
				//NSLog(@"Flowing @ %f %x %x %x %x %x %x %x!",time, pyramid, prevPyramid, autoPrevFrame, frame, pointStatus, points, prevPoints);
				if(pointCount)
					cvCalcOpticalFlowPyrLK( autoPrevFrame, frame, prevPyramid, pyramid,
								 prevPoints, points,
								 pointCount, cvSize([inputWindowSize indexValue],[inputWindowSize indexValue]), 3, pointStatus,
								 NULL, cvTermCriteria (CV_TERMCRIT_ITER|CV_TERMCRIT_EPS,
									   [inputMaxIterations indexValue], [inputEpsilon doubleValue]), 0);
				float nan = 0.0;
				nan /= nan;
				//[pointStruct removeAllMembers];
				pointStruct = [[QCStructure alloc] init];
				float aspect = frame->height / (float)frame->width;
				for(i=0;i<pointCount;++i)
				{
					QCStructure *outPoint = [[QCStructure allocWithZone:NULL] init];
					if(pointStatus[i])
					{
						[outPoint addMember:[NSNumber numberWithFloat:CV2QC(points[i].x, frame->width)] forKey:@"X"];
						[outPoint addMember:[NSNumber numberWithFloat:CV2QC(points[i].y, frame->height) * aspect] forKey:@"Y"];
						[outPoint addMember:[NSNumber numberWithFloat:CV2QC(prevPoints[i].x, frame->width)] forKey:@"pX"];
						[outPoint addMember:[NSNumber numberWithFloat:CV2QC(prevPoints[i].y, frame->height) * aspect] forKey:@"pY"];
						[pointStruct addMember:outPoint forKey:[NSString stringWithFormat:@"Point%04i",i]];
					}
					else if([inputExcludeNaNs booleanValue] == FALSE)
					{
						[outPoint addMember:[NSNumber numberWithFloat:nan] forKey:@"X"];
						[outPoint addMember:[NSNumber numberWithFloat:nan] forKey:@"Y"];
						[outPoint addMember:[NSNumber numberWithFloat:nan] forKey:@"pX"];
						[outPoint addMember:[NSNumber numberWithFloat:nan] forKey:@"pY"];
						[pointStruct addMember:outPoint forKey:[NSString stringWithFormat:@"Point%04i",i]];
					}
					
					[outPoint release];
				}
				// swap all the frame data (pyramid, frame, points)
				cvCopy(frame, autoPrevFrame, 0);
				
				void *tmp = points;
				points = prevPoints;
				prevPoints = tmp;
				tmp = pyramid;
				pyramid = prevPyramid;
				prevPyramid = tmp;
				[outputPoints setStructureValue: pointStruct];
				[pointStruct release];
			}
			else	// make image, then copy points to output points
			{
				//NSLog(@"duplicating frame @ %f",time);
				if(autoPrevFrame)	// resized, discard this version
					cvReleaseImage(&autoPrevFrame);
				autoPrevFrame = cvCreateImage(cvGetSize(frame), frame->depth, frame->nChannels);
				cvCopy(frame, autoPrevFrame, 0);
			}
		}
	}
	else	// no input...
	{
		if(autoPrevFrame)
		{
			cvReleaseImage(&autoPrevFrame);
			autoPrevFrame = NULL;
		}
		[outputPoints setStructureValue:nil];
	}

	return YES;
}

@end
