#import "OpenCVFindCornerSubPix.h"

@implementation OpenCVFindCornerSubPix : QCPatch

+(BOOL)isSafe
{
	return YES;
}

+ (BOOL)allowsSubpatchesWithIdentifier:(id)fp8
{
	return NO;
}

- (id)initWithIdentifier:(id)fp8
{
	if(self = [super initWithIdentifier:fp8])
	{
		[inputEpsilon setMinDoubleValue: 0.0];
		[inputEpsilon setMaxDoubleValue: 5.0];
		[inputEpsilon setDoubleValue: 0.03];
		
		[inputMaxIterations setMaxIndexValue: 50];
		[inputMaxIterations setIndexValue: 25];
		
		[inputZeroZone setMaxIndexValue: 10];
		[inputWindowSize setMaxIndexValue: 50];
		[inputWindowSize setIndexValue: 10];
				
		[inputImage setType: OPENCV_IMAGE];
		
		pointSize = 1024;	// 1024 points initially.  'enough for anybody'
		points = (CvPoint2D32f*)malloc(sizeof(CvPoint2D32f)*pointSize);
		//outPoints = [[QCStructure alloc] init];
	}

	return self;
}

- (void)dealloc
{
	if(points)
		free(points);
	[super dealloc];
}

- (BOOL)execute:(QCOpenGLContext *)context time:(double)time arguments:(NSDictionary *)arguments
{
	IplImage *frame = [[inputImage cvObjectValue] value];
	QCStructure *inPoints = [inputPoints structureValue];
	if(frame && inPoints)
	{
		if([inputFindSignal booleanValue] && !oldFindSignal)
		{
			unsigned int i, count;
			if([inPoints count] > pointSize)	// too many points, need to grow the point array
			{
				while(pointSize < [inPoints count])
					pointSize += 256;	// add room for 256 more points
				points = (CvPoint2D32f*)realloc(points,pointSize*sizeof(CvPoint2D32f));
			}
			// read out points, refine corners, and then post output
			//[outPoints removeAllMembers];
			outPoints = [[QCStructure alloc] init];
			count = [inPoints count];
			for(i=0;i<count;i++)
			{
				QCStructure *point = [inPoints memberForKey:[NSString stringWithFormat:@"Point%04i",i]];
				points[i].x = QC2CV([[point memberForKey:@"X"] doubleValue], frame->width);
				points[i].y = QC2CV([[point memberForKey:@"Y"] doubleValue]* frame->width/frame->height, frame->height);
			}
			cvFindCornerSubPix(frame, points, count, 
					cvSize([inputWindowSize indexValue], [inputWindowSize indexValue]),
					cvSize([inputZeroZone indexValue]-1, [inputZeroZone indexValue]-1), 
					cvTermCriteria (CV_TERMCRIT_ITER|CV_TERMCRIT_EPS,
                                   [inputMaxIterations indexValue], [inputEpsilon doubleValue]));
			for(i=0;i<count;++i)
			{
				QCStructure *point = [[QCStructure allocWithZone:NULL] init];
				[point addMember:[NSNumber numberWithDouble:CV2QC(points[i].x, frame->width)] forKey:@"X"];
				[point addMember:[NSNumber numberWithDouble:CV2QC(points[i].y, frame->height) * frame->height/frame->width] forKey:@"Y"];
				[outPoints addMember: point forKey:[NSString stringWithFormat:@"Point%04i",i]];
				[point release];
			}
			[outputPoints setStructureValue: outPoints];
			[outPoints release];
		}
		oldFindSignal = [inputFindSignal booleanValue];
	}
	else	// no input
	{
		[outputPoints setStructureValue: nil];
	}

	return YES;
}

@end
