#import "OpenCVGoodFeaturesToTrack.h"

@implementation OpenCVGoodFeaturesToTrack : QCPatch

+(BOOL)isSafe
{
	return YES;
}

+ (BOOL)allowsSubpatchesWithIdentifier:(id)fp8
{
	return NO;
}

- (id)initWithIdentifier:(id)fp8
{
	if(self = [super initWithIdentifier:fp8])
	{
		[inputMaxPoints setMaxIndexValue:1000];
		[inputMaxPoints setIndexValue:50];
		[inputQuality setMaxDoubleValue: 1.0];
		[inputQuality setMinDoubleValue: 0.0];
		[inputQuality setDoubleValue: 0.01];
		[inputMinDistance setMaxDoubleValue:32];
		[inputMinDistance setMinDoubleValue:1];
		[inputMinDistance setDoubleValue: 10.0];
		[inputImage setType: OPENCV_IMAGE];
		points = [QCStructure new];
	}

	return self;
}

- (void)dealloc
{
	if(eig)
		cvReleaseImage(&eig);
	if(temp)
		cvReleaseImage(&temp);
	[points release];
	[super dealloc];
}

- (void)disable:(QCOpenGLContext *)context
{
	oldFindValue = FALSE;
}


- (BOOL)execute:(QCOpenGLContext *)context time:(double)time arguments:(NSDictionary *)arguments
{
	IplImage *inImg = [[inputImage cvObjectValue] value];
	if(inImg && inImg->nChannels == 1 && [inputMaxPoints indexValue] > 0)
	{
		if([inputFindSignal booleanValue] && !oldFindValue)
		{
			unsigned int i;
			CvPoint2D32f *pointSeq = (CvPoint2D32f*)malloc(sizeof(CvPoint2D32f)*[inputMaxPoints indexValue]);
			[points removeAllMembers];
			if(eig == NULL || eig->width != inImg->width || eig->height != inImg->height)
			{
				if(eig)
					cvReleaseImage(&eig);
				eig = cvCreateImage(cvGetSize(inImg), 32, 1);
				cvSetZero(eig);
			}
			if(temp == NULL || temp->width != inImg->width || temp->height != inImg->height)
			{
				if(temp)
					cvReleaseImage(&temp);
				temp = cvCreateImage(cvGetSize(inImg), 32, 1);
				cvSetZero(temp);
			}
			int cornerCount = [inputMaxPoints indexValue];
			cvGoodFeaturesToTrack(inImg, eig, temp, pointSeq,
				&cornerCount,
				[inputQuality doubleValue],
				[inputMinDistance doubleValue],
				NULL, 7, 0, 0.04);
			for(i=0;i<cornerCount;++i)
			{
				QCStructure *pointStruct = [[QCStructure allocWithZone:NULL] init];
				//NSLog(@"Point: %f -> %f -> %f",pointSeq[i].x, CV2QC(pointSeq[i].x, inImg), QC2CV( CV2QC(pointSeq[i].x, inImg), inImg ));
				// 
				[pointStruct addMember:[NSNumber numberWithDouble: CV2QC(pointSeq[i].x, inImg->width) ] forKey:@"X"];
				[pointStruct addMember:[NSNumber numberWithDouble: CV2QC(pointSeq[i].y, inImg->height)*inImg->height/inImg->width ] forKey:@"Y"];
				[points addMember: pointStruct forKey:[NSString stringWithFormat:@"Point%04i",i]];
				[pointStruct release];
			}
		
			[outputPoints setStructureValue:points];
		
			free(pointSeq);
		}
		oldFindValue = [inputFindSignal booleanValue];
	}
	else	// disconnected/zero points.  output null
	{
		cvReleaseImage(&eig);
		cvReleaseImage(&temp);
		eig = NULL;
		temp = NULL;
		[outputPoints setStructureValue: nil];
		oldFindValue = FALSE;
		if([points count])	// only realloc if it has stuff in it (to reset the structure)
		{
			[points release];
			points = [QCStructure new];
		}
	}

	return YES;
}

@end
