#import "OpenCVHaarDetectObjects.h"


@implementation OpenCVHaarDetectObjects : QCPatch

+(BOOL)isSafe
{
	return YES;
}

+ (BOOL)allowsSubpatchesWithIdentifier:(id)fp8
{
	return NO;
}

- (id)initWithIdentifier:(id)fp8
{
	if(self=[super initWithIdentifier:fp8])
	{
		[inputImage setType: OPENCV_IMAGE];
		[inputCascade setType: OPENCV_HAAR];
		
		[inputImageScale setDoubleValue: 2.1];
		[inputImageScale setMinDoubleValue: 1.0];
		[inputScaleFactor setMinDoubleValue: 1.05];
		[inputScaleFactor setMaxDoubleValue: 6.0];
		[inputScaleFactor setDoubleValue: 1.3];
		
		[inputMinNeighbors setIndexValue: 1];
		[inputMinSize setIndexValue: 20];
		[inputCannyPruning setBooleanValue: YES];
		storage = cvCreateMemStorage(0);
	}

	return self;
}

- (void)dealloc
{
	//[rects release];
	if(storage)
		cvReleaseMemStorage(&storage);
	[super dealloc];
}

- (BOOL)execute:(QCOpenGLContext *)context time:(double)time arguments:(NSDictionary *)arguments
{
	IplImage *frame = [[inputImage cvObjectValue] value];
	IplImage *small, *gray;
	CvHaarClassifierCascade *cascade = [[inputCascade cvObjectValue] value];

	if(frame && cascade)
	{
		unsigned int i;
		float imageScale = [inputImageScale doubleValue];
		rects = [[QCStructure alloc] init];
		
		gray = cvCreateImage(cvSize(frame->width, frame->height), 8, 1);
		small = cvCreateImage(cvSize(cvRound(frame->width/imageScale), cvRound(frame->height/imageScale)), 8, 1);
		
		if(frame->nChannels == 3)
			cvCvtColor(frame, gray, CV_BGR2GRAY);
		else if(frame->nChannels == 4)
			cvCvtColor(frame, gray, CV_BGRA2GRAY);
		else if(frame->nChannels == 1)
			cvCopy(frame, gray, NULL);
		cvResize(gray, small, CV_INTER_LINEAR);
		// flip it upside down, since haar detection is orientation dependent
		cvFlip(small, small, 0);
		cvEqualizeHist(small, small);
		//cvResize(gray, small, CV_INTER_LINEAR);
		
		cvClearMemStorage(storage);
		
		//NSLog(@"image: %i %i %i",small->width, small->height, small->nChannels);
		//cvSaveImage("/Users/cwright/Desktop/small.plugin.png",small);
		//cvSaveImage("/Users/cwright/Desktop/gray.plugin.png",gray);
		float aspect = small->height/(float)small->width;
		rectSeq = cvHaarDetectObjects( small, cascade,
					storage, [inputScaleFactor doubleValue],
					[inputMinNeighbors indexValue], (([inputCannyPruning booleanValue]==YES)?CV_HAAR_DO_CANNY_PRUNING:0),
					cvSize([inputMinSize indexValue],[inputMinSize indexValue]) );
		// test call -- ignore user inputs for sanity
		//rectSeq = cvHaarDetectObjects(small, cascade, storage,
		//							  1.2, 2, CV_HAAR_DO_CANNY_PRUNING/*|CV_HAAR_FIND_BIGGEST_OBJECT*/, cvSize(20,20));
		//NSLog(@"Detected %i objects @ %f",rectSeq->total/4, time);
		for(i=0; i< (rectSeq?rectSeq->total:0);++i)
		{
			CvRect *r = (CvRect*)cvGetSeqElem(rectSeq, i);
			QCStructure *objStruct = [[QCStructure allocWithZone:NULL] init];
			//NSLog(@"r->height: %f r->width: %f",(float)r->height, (float)r->width);
			[objStruct addMember:[NSNumber numberWithFloat: CV2QC(r->x+r->width/2.,small->width)] forKey:@"X"];
			[objStruct addMember:[NSNumber numberWithFloat: -CV2QC(r->y+r->height/2.,small->height)*aspect] forKey:@"Y"];
			[objStruct addMember:[NSNumber numberWithFloat: 2.0*r->width/(float)small->width/*(CV2QC(r->width,small->width))*/] forKey:@"W"];
			[objStruct addMember:[NSNumber numberWithFloat: 2.0*r->height/(float)small->height/*(CV2QC(r->height,small->height))*aspect*/] forKey:@"H"];
			//[objStruct addMember:[NSNumber numberWithDouble:CV2QC(r->x+r->width,small->width)] forKey:@"X2"];
			//[objStruct addMember:[NSNumber numberWithDouble:CV2QC(r->y+r->height,small->height)*small->height/small->width] forKey:@"Y2"];
			[rects addMember: objStruct forKey:[NSString stringWithFormat:@"Object%04i",i]];
			[objStruct release];
		}
		cvReleaseImage(&gray);
		cvReleaseImage(&small);

		[outputRects setStructureValue: rects];
		[rects release];
	}
	else	// missing useful input
	{
		//[rects removeAllMembers];
		rects = nil;
		[outputRects setStructureValue:nil];
	}

	return YES;
}

@end
