#import "OpenCVHoughLines2.h"

@implementation OpenCVHoughLines2 : QCPatch

+(BOOL)isSafe
{
	return YES;
}

+ (BOOL)allowsSubpatchesWithIdentifier:(id)fp8
{
	return NO;
}

- (id)initWithIdentifier:(id)fp8
{
	if(self = [super initWithIdentifier:fp8])
	{
		[inputImage setType: OPENCV_IMAGE];
		
		[inputRho setDoubleValue: 1.0];
		[inputRho setMinDoubleValue: 0.5];
		[inputTheta setDoubleValue: 1.0];
		[inputTheta setMaxDoubleValue: 179.9];
		[inputTheta setMinDoubleValue: 0.1];
		
		[inputRho setDoubleValue: 0.0];
		
		[inputThreshold setIndexValue: 25];
		
		[inputMinLength setDoubleValue: 50];
		[inputMaxGap setDoubleValue: 15];
		
		storage = cvCreateMemStorage(0);
		lines = [QCStructure new];
	}

	return self;
}

- (BOOL)execute:(QCOpenGLContext *)context time:(double)time arguments:(NSDictionary *)arguments
{
	IplImage *frame = [[inputImage cvObjectValue] value];
	
	if(frame && frame->nChannels == 1)
	{
		unsigned int i;
		[lines removeAllMembers];
		cvClearMemStorage(storage);
		lineSeq = cvHoughLines2(frame, storage, CV_HOUGH_PROBABILISTIC, [inputRho doubleValue], [inputTheta doubleValue] * CV_PI/180.0,
					([inputThreshold indexValue]>0?[inputThreshold indexValue]:1), [inputMinLength doubleValue], [inputMaxGap doubleValue]);
		//NSLog(@"Found %i lines",lineSeq->total);
		for(i=0;i<(lineSeq?lineSeq->total:0);++i)
		{
			QCStructure *lineStruct = [[QCStructure allocWithZone:NULL] init];
			CvPoint* line = (CvPoint*)cvGetSeqElem(lineSeq,i);
			[lineStruct addMember:[NSNumber numberWithDouble:CV2QC(line[0].x, frame->width)]  forKey:@"X1"];
			[lineStruct addMember:[NSNumber numberWithDouble:CV2QC(line[0].y, frame->height)*frame->height/frame->width] forKey:@"Y1"];
			[lineStruct addMember:[NSNumber numberWithDouble:CV2QC(line[1].x, frame->width)]  forKey:@"X2"];
			[lineStruct addMember:[NSNumber numberWithDouble:CV2QC(line[1].y, frame->height)*frame->height/frame->width] forKey:@"Y2"];
			[lines addMember: lineStruct forKey:[NSString stringWithFormat:@"Line%04i",i]];
			[lineStruct release];
		}
		[outputLines setStructureValue: lines];
	}
	else	// invalid/missing input
	{
		[lines removeAllMembers];
		[outputLines setStructureValue: nil];
	}

	return YES;
}

@end
