#import "OpenCVImageToQCImage.h"

@implementation OpenCVImageToQCImage : QCPatch

+(BOOL)isSafe
{
	return YES;
}

+ (BOOL)allowsSubpatchesWithIdentifier:(id)fp8
{
	return NO;
}

- (id)initWithIdentifier:(id)fp8
{
	if(self = [super initWithIdentifier:fp8])
	{
		[inputImage setType: OPENCV_IMAGE];
	}

	return self;
}

- (void)dealloc
{
	if(cvt)
		cvReleaseImage(&cvt);
	[super dealloc];
}

- (void)enable:(QCOpenGLContext *)context
{
	cs = CGColorSpaceCreateWithName(kCGColorSpaceGenericRGB);
	pf = [QCPixelFormat pixelFormatBGRX8];
}
- (void)disable:(QCOpenGLContext *)context
{
	CGColorSpaceRelease(cs);
}


- (BOOL)execute:(QCOpenGLContext *)context time:(double)time arguments:(NSDictionary *)arguments
{
	IplImage *img = [[inputImage cvObjectValue] value];
	if(img)
	{
		if( cvt == NULL || cvt->width != img->width || cvt->height != img->height)
		{
			if(cvt)
				cvReleaseImage(&cvt);
			cvt = cvCreateImage(cvGetSize(img), 8, 4);
		}
		if (img->nChannels == 4)
		{
			cvCopy(img, cvt, 0);
			//cvCvtColor(img, cvt, CV_BGRA2RGBA);
			//cvFlip(cvt,cvt,-1);
		}
		if(img->nChannels == 3)
			cvCvtColor(img, cvt, CV_BGR2BGRA);
		else if (img->nChannels == 1)
			cvCvtColor(img, cvt, CV_GRAY2BGRA);
		//[pb release];
		pb = [[context imageManager]
				createPixelBufferWithFormat: pf
				pixelsWide: cvt->width
				pixelsHigh: cvt->height
				options: nil ];
		IplImage fImg;
		[pb beginUpdatePixels:TRUE colorSpace:cs];
		//memcpy([pb baseAddress], cvt->imageData, cvt->width*cvt->height*4);
		/* rather than memcpy, we use cv's width-handling stuff in a fake IplImage for better CIImage integration etc.*/
		fImg.nSize = sizeof(IplImage);
		//fImg.ID = 0;
		fImg.imageData = [pb baseAddress];
		fImg.nChannels = 4;
		fImg.depth = 8;
		fImg.dataOrder = 0;
		fImg.width = cvt->width;
		fImg.height = cvt->height;
		fImg.roi = NULL;
		//fImg.maskROI = NULL;
		fImg.imageSize = fImg.height * [pb bytesPerRow];
		fImg.widthStep = [pb bytesPerRow];
		fImg.imageDataOrigin = fImg.imageData;
		cvCopy(cvt, &fImg,0);
		[pb endUpdatePixels];

		QCImage *image=[[QCImage allocWithZone:NULL] initWithQCImageBuffer:pb options:nil];
		// disable color correction.  might accelerate it a bit (XXX need to profile with and without)
		[image setMetadata:[NSNumber numberWithBool:TRUE] forKey:@"disableColorMatching" shouldForward: NO];
		
		[outputImage setImageValue:image];
		[image release];
		[pb release];
		//cvReleaseImage(&cvt);
	}
	else	// nothing connected
	{
		[outputImage setImageValue:nil];
	}

	return YES;
}

@end
