#import "OpenCVLogPolar.h"

@implementation OpenCVLogPolar : QCPatch

+(BOOL)isSafe
{
	return YES;
}

+ (BOOL)allowsSubpatchesWithIdentifier:(id)fp8
{
	return NO;
}

- (id)initWithIdentifier:(id)fp8
{
	if(self = [super initWithIdentifier:fp8])
	{
		[inputMagnitude setMinDoubleValue: 0.01];
		[inputMagnitude setDoubleValue: 40];
		[inputWidth setIndexValue: 256];
		[inputHeight setIndexValue: 256];
		[inputImage setType: OPENCV_IMAGE];
		[outputImage setType: OPENCV_IMAGE];
	}

	return self;
}

- (void)disable:(QCOpenGLContext *)context
{
	if(img)
	{
		cvReleaseImage(&img);
		img = NULL;
	}
}


- (BOOL)execute:(QCOpenGLContext *)context time:(double)time arguments:(NSDictionary *)arguments
{
	IplImage *frame = [[inputImage cvObjectValue] value];
	if(frame && [inputWidth indexValue] && [inputHeight indexValue])
	{
		if(img == NULL || [inputWidth indexValue] != img->width || [inputHeight indexValue] != img->height || frame->nChannels != img->nChannels)
		{
			if(img)
				cvReleaseImage(&img);
			//img = cvCreateImage(cvGetSize(frame), frame->depth, frame->nChannels);
			img = cvCreateImage(cvSize([inputWidth indexValue], [inputHeight indexValue]), frame->depth, frame->nChannels);
		}
		cvLogPolar(frame, img, cvPoint2D32f([inputCenterX doubleValue],[inputCenterY doubleValue]), [inputMagnitude doubleValue], 
			CV_INTER_LINEAR|CV_WARP_FILL_OUTLIERS | ([inputInverse booleanValue] * CV_WARP_INVERSE_MAP));

		OpenCVObject *obj = [[OpenCVObject allocWithZone:NULL] initWithValue:img];
		[outputImage setCvObjectValue: obj];
		[obj release];
	}
	else	// no input image
	{
		if(img)
		{
			cvReleaseImage(&img);
			img = NULL;
		}
		[outputImage setCvObjectValue: nil];
	}

	return YES;
}

@end
