#import "OpenCVResize.h"

@implementation OpenCVResize : QCPatch

+(BOOL)isSafe
{
	return YES;
}

+ (BOOL)allowsSubpatchesWithIdentifier:(id)fp8
{
	return NO;
}

- (id)initWithIdentifier:(id)fp8
{
	if(self = [super initWithIdentifier:fp8])
	{
		[inputImage setType: OPENCV_IMAGE];
		[outputImage setType: OPENCV_IMAGE];
		
		[inputWidth setMaxIndexValue: 2048];
		[inputWidth setIndexValue: 320];
		[inputHeight setMaxIndexValue: 2048];
		[inputHeight setIndexValue: 240];
		
		[inputMethod setMaxIndexValue: 3];
		[inputMethod setIndexValue: CV_INTER_LINEAR];
	}

	return self;
}

- (void)dealloc
{
	if(img)
		cvReleaseImage(&img);
	[super dealloc];
}

- (BOOL)execute:(QCOpenGLContext *)context time:(double)time arguments:(NSDictionary *)arguments
{
	IplImage *frame = [[inputImage cvObjectValue] value];
	if(frame && [inputWidth indexValue] && [inputHeight indexValue])
	{
		if( img == NULL || [inputWidth indexValue] != width || [inputHeight indexValue] != height ||
			img->depth != frame->depth || img->nChannels != frame->nChannels)
		{
			// reallocate image since dimensions/parameters changed
			if(img)
				cvReleaseImage(&img);
			img = cvCreateImage(cvSize([inputWidth indexValue],[inputHeight indexValue]), frame->depth, frame->nChannels);
		}
		cvResize(frame, img, [inputMethod indexValue]);
		OpenCVObject *obj = [[OpenCVObject allocWithZone:NULL] initWithValue:img];
		[outputImage setCvObjectValue: obj];
		[obj release];
	}
	else	// no input
	{
		if(img)
		{
			cvReleaseImage(&img);
			img = NULL;
		}
		[outputImage setCvObjectValue: nil];
	}
	width = [inputWidth indexValue];
	height = [inputHeight indexValue];

	return YES;
}

@end
