#import "QCImageToOpenCVImage.h"
//#import <OpenGL/CGLMacro.h>

@implementation QCImageToOpenCVImage : QCPatch

+(BOOL)isSafe
{
	return YES;
}

+ (BOOL)allowsSubpatchesWithIdentifier:(id)fp8
{
	return NO;
}

- (id)initWithIdentifier:(id)fp8
{
	if(self = [super initWithIdentifier:fp8])
	{
		[outputImage setType: OPENCV_IMAGE];
		[inputFlip setMaxIndexValue: 3];
	}

	return self;
}

- (void)dealloc
{	
	if(img)
		cvReleaseImage(&img);
	[super dealloc];
}

- (BOOL)execute:(QCOpenGLContext *)context time:(double)time arguments:(NSDictionary *)arguments
{
	QCImage *inImg = [inputImage imageValue];

	if(inImg && [inImg isInfinite] == FALSE)
	{
		//CGLContextObj cgl_ctx = [context CGLContextObj];
		NSRect bounds = [inImg bounds];
		//NSLog(@"image provider: %@",[inImg provider]);
		//NSLog(@"pixel format: %@",[[inImg provider] nativePixelFormat]);
		//NSLog(@"QC2CV: img: %x",inImg);
		//NSLog(@"QC2CV: static meta: %@",[inImg allStaticMetadata]);
		//NSLog(@"QC2CV: fwd meta:    %@",[inImg allForwardedMetadata]);
		if(img == NULL || img->width != bounds.size.width || img->height != bounds.size.height)
		{
			if(img)
				cvReleaseImage(&img);
			img = cvCreateImage(cvSize((int)bounds.size.width,(int)bounds.size.height),8,4);
		}
		IplImage fImg;
		id mgr = [context imageManager];
		//NSBitmapImageRep *bir = [inImg createNSBitmapImageRepForManager: mgr withOptions:nil];
		NSBitmapImageRep *bir = [inImg createRepresentationOfType:@"NSBitmapImageRep" forManager:mgr withOptions:nil];
		
		//fImg.ID = 0;
		fImg.imageData = (char*)[bir bitmapData];//[pb baseAddress];
		fImg.nSize = sizeof(IplImage);
		fImg.nChannels = 4;
		fImg.depth = 8;
		fImg.dataOrder = 0;
		fImg.width = bounds.size.width;
		fImg.height = bounds.size.height;
		fImg.roi = NULL;
		//fImg.maskROI = NULL;
		fImg.widthStep = [bir bytesPerRow];//[pb bytesPerRow];
		fImg.imageSize = fImg.height * fImg.widthStep;//[pb bytesPerRow];
		fImg.imageDataOrigin = fImg.imageData;

		switch([inputFlip indexValue])
		{
			case 0:
			default:
				cvFlip(&fImg, img, 0);
				break;
			case 1:	// flip horizontally
				cvFlip(&fImg, img, -1);
				break;
			case 2:	// flip vertically
				cvCopy(&fImg, img, 0);
				break;
			case 3:	// flip horizontally+vertically
				cvFlip(&fImg, img, 1);
				break;
		}
		
		//cvCopy(&fImg, img, 0);
		OpenCVObject *obj = [[OpenCVObject allocWithZone:NULL] initWithValue: img];
		[outputImage setCvObjectValue: obj];
		[obj release];
		
		[bir release];
	}
	else	// nil or infinite input
	{
		if(img)
		{
			cvReleaseImage(&img);
			img = NULL;
		}
		[outputImage setCvObjectValue:nil];
	}

	return YES;
}

@end
